import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class ELastEvent(Enum):
  BELOW=200
  IN_RANGE=201
  ABOVE=202
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in ELastEvent.__members__.values():
      if (act.value == checkValue):
        return act

    return ELastEvent.SER_UNKNOWN






from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.pIDController.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.pIDController.params.EEnable import EEnable
from pyhausbus.de.hausbus.homeassistant.proxy.pIDController.params.EErrorCode import EErrorCode

class PIDController(ABusFeature):
  CLASS_ID:int = 44

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return PIDController(HausBusUtils.getObjectId(deviceId, 44, instanceId))

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param P P-Anteil des Reglers.
  @param I I-Anteil des Reglers.
  @param D D-Anteil des Reglers.
  @param targetValue Regelungszielwert z.B. targetValue*0.
  @param sensorObjectId Komplette Objekt-ID des Feedback-Sensors.
  @param actorObjectId Komplette Objekt-ID des Stellers.
  @param timeout Zeit.
  @param hysteresis Erweitert den Regelzielwert in einen Bereich\r\n0: Regelzielwert wird versucht exakt zu erreichen\r\n>0: Regelzielwert +/- hysteresis wird versucht zu erreichen.
  """
  def setConfiguration(self, P:int, I:int, D:int, targetValue:int, sensorObjectId:int, actorObjectId:int, timeout:int, hysteresis:int):
    LOGGER.debug("setConfiguration"+" P = "+str(P)+" I = "+str(I)+" D = "+str(D)+" targetValue = "+str(targetValue)+" sensorObjectId = "+str(sensorObjectId)+" actorObjectId = "+str(actorObjectId)+" timeout = "+str(timeout)+" hysteresis = "+str(hysteresis))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addWord(P)
    hbCommand.addWord(I)
    hbCommand.addWord(D)
    hbCommand.addWord(targetValue)
    hbCommand.addDWord(sensorObjectId)
    hbCommand.addDWord(actorObjectId)
    hbCommand.addWord(timeout)
    hbCommand.addByte(hysteresis)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param P P-Anteil des Reglers.
  @param I I-Anteil des Reglers.
  @param D D-Anteil des Reglers.
  @param targetValue Regelungszielwert z.B. targetValue*0.
  @param sensorObjectId Komplette Objekt-ID des Feedback-Sensors.
  @param actorObjectId Komplette Objekt-ID des Stellers.
  @param timeout Zeit.
  @param hysteresis Erweitert den Regelzielwert in einen Bereich\r\n0: Regelzielwert wird versucht exakt zu erreichen\r\n>0: Regelzielwert +/- hysteresis wird versucht zu erreichen.
  """
  def Configuration(self, P:int, I:int, D:int, targetValue:int, sensorObjectId:int, actorObjectId:int, timeout:int, hysteresis:int):
    LOGGER.debug("Configuration"+" P = "+str(P)+" I = "+str(I)+" D = "+str(D)+" targetValue = "+str(targetValue)+" sensorObjectId = "+str(sensorObjectId)+" actorObjectId = "+str(actorObjectId)+" timeout = "+str(timeout)+" hysteresis = "+str(hysteresis))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addWord(P)
    hbCommand.addWord(I)
    hbCommand.addWord(D)
    hbCommand.addWord(targetValue)
    hbCommand.addDWord(sensorObjectId)
    hbCommand.addDWord(actorObjectId)
    hbCommand.addWord(timeout)
    hbCommand.addByte(hysteresis)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param targetValue Regelungszielwert z.B. targetValue*0.
  """
  def setTargetValue(self, targetValue:int):
    LOGGER.debug("setTargetValue"+" targetValue = "+str(targetValue))
    hbCommand = HausBusCommand(self.objectId, 2, "setTargetValue")
    hbCommand.addWord(targetValue)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param enable Reglerverhalten ein/ausschalten.
  """
  def enable(self, enable:EEnable):
    LOGGER.debug("enable"+" enable = "+str(enable))
    hbCommand = HausBusCommand(self.objectId, 3, "enable")
    hbCommand.addByte(enable.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOn(self):
    LOGGER.debug("evOn")
    hbCommand = HausBusCommand(self.objectId, 200, "evOn")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOff(self):
    LOGGER.debug("evOff")
    hbCommand = HausBusCommand(self.objectId, 201, "evOff")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EIndex import EIndex
import pyhausbus.HausBusUtils as HausBusUtils

class GetModuleId:
  CLASS_ID = 0
  FUNCTION_ID = 2

  def __init__(self,index:EIndex):
    self.index=index


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetModuleId(EIndex._fromBytes(dataIn, offset))

  def __str__(self):
    return f"GetModuleId(index={self.index})"

  '''
  @param index .
  '''
  def getIndex(self):
    return self.index






from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.EEnable import EEnable
import pyhausbus.HausBusUtils as HausBusUtils

class EnableEvents:
  CLASS_ID = 16
  FUNCTION_ID = 2

  def __init__(self,enable:EEnable, disabledDuration:int):
    self.enable=enable
    self.disabledDuration=disabledDuration


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EnableEvents(EEnable._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"EnableEvents(enable={self.enable}, disabledDuration={self.disabledDuration})"

  '''
  @param enable FALSE: Deaktiviert das Versenden von Events\r\nTRUE: Aktiviert das Versenden von Events\r\nINVERT: Invertiert das aktuelle Verhalten.
  '''
  def getEnable(self):
    return self.enable

  '''
  @param disabledDuration Zeit1s-255s f?  ? ? ?r die die Events deaktiviert werden sollen 0 = unendlich \r\nDieser Parameter wirkt nur.
  '''
  def getDisabledDuration(self):
    return self.disabledDuration






from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.ObjectId import ObjectId

class ABusFeature:

  def __init__(self, objectId:int):
    self.objectId = objectId
    self.name:str=""

  def getObjectId(self) -> int:
    return self.objectId

  def getName(self) -> str:
    return self.name

  def setName(self, name:str):
      self.name = name

  def __str__(self):
    dummy, classType = str(type(self)).rsplit(".", 1)
    classType = classType[:-2]
    return f"ABusFeature(type={classType}, name={self.name}, objectId={ObjectId(self.objectId)}"



import socket
import threading
import traceback
from pyhausbus.HausBusUtils import *
from pyhausbus.de.hausbus.homeassistant.proxy import *
import time

BROADCAST_SEND_IP = "192.255.255.255"
BROADCAST_RECEIVE_IP = "0.0.0.0"
BUFFER_SIZE  = 10000

class UdpReceiveWorker:
  UDP_GATEWAY = "#UDP#"

  def __init__(self, func):
    LOGGER.debug("init UdpReceiveWorker")
    self.func = func

  def startWorker(self):
    LOGGER.debug("starting udp receive worker")
    t = threading.Thread(target=self.runable)
    t.start()

  def runable(self):
    while(True):
      try:
        UDPServerSocket = socket.socket(family=socket.AF_INET, type=socket.SOCK_DGRAM)
        UDPServerSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        UDPServerSocket.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        UDPServerSocket.bind((BROADCAST_RECEIVE_IP, UDP_PORT))
        LOGGER.debug("UDP server up and listening")

        while(True):
          bytesAddressPair = UDPServerSocket.recvfrom(BUFFER_SIZE)
          message = bytesAddressPair[0]
          address = bytesAddressPair[1]
          LOGGER.debug("Message from Client "+format(address)+": "+bytesToDebugString(message))


          if (len(message) == 0):
            LOGGER.debug("got empty message")
            continue

          if (message[0] != 0xef | message[1] != 0xef):
            LOGGER.debug("invalid header")
            continue

          if (len(message) < 15):
            LOGGER.debug("message size " + len(message) + " is too short")
            continue

          # 2 = Kontrollbyte 3 = MessageCounter
          offset = [4]
          senderObjectId = bytesToDWord(message, offset)
          LOGGER.debug("senderObjectId = "+str(senderObjectId))

          receiverObjectId = bytesToDWord(message, offset)
          LOGGER.debug("receiverObjectId = "+str(receiverObjectId))

          dataLength = bytesToWord(message, offset)
          if (len(message) < 14 + dataLength):
            LOGGER.debug("message size " + str(len(message)) + " is too short for data length " + str(dataLength) + ": " + bytesToDebugString(message))
            dataLength = len(message) - 14
            # continue
          functionId = bytesToInt(message, offset)
          functionData = message[15:]

          LOGGER.debug("functionId " + str(functionId) + ", functionData " + bytesToDebugString(functionData))

          self.func(senderObjectId, receiverObjectId, functionId, functionData, self.UDP_GATEWAY, False)
      except (Exception) as err:
        LOGGER.error(err,exc_info=True,stack_info=True)
        time.sleep(5)



import pyhausbus.HausBusUtils as HausBusUtils

class Enabled:
  CLASS_ID = 16
  FUNCTION_ID = 130

  def __init__(self,enabled:int):
    self.enabled=enabled


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Enabled(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"Enabled(enabled={self.enabled})"

  '''
  @param enabled 0: Events sind deaktviert\r\n1: Events sind aktiviert.
  '''
  def getEnabled(self):
    return self.enabled






from pyhausbus.WeekTime import WeekTime
import pyhausbus.HausBusUtils as HausBusUtils

class SetTime:
  CLASS_ID = 0
  FUNCTION_ID = 125

  def __init__(self,weekTime:WeekTime):
    self.weekTime=weekTime


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetTime(WeekTime._fromBytes(dataIn, offset))

  def __str__(self):
    return f"SetTime(weekTime={self.weekTime})"

  '''
  @param weekTime .
  '''
  def getWeekTime(self):
    return self.weekTime






import pyhausbus.HausBusUtils as HausBusUtils

class WeekTime:

  def __init__(self, day:int, hour:int, minute:int):
    self.day = day
    self.hour = hour
    self.minute = minute

  @staticmethod
  def fromBytes(data: bytearray, offset: int) -> 'WeekTime':
    value = HausBusUtils.bytesToWord(data, offset)

    minute = value & 0xff
    value = value >> 8
    hour = value & 0x1F
    day = value >> 5
    return WeekTime(day, hour, minute)

  def getValue(self) -> int:
    value = self.day << 5
    value += self.hour
    value = value << 8
    value += self.minute
    return value

  @staticmethod
  def fromValue(value:int) -> 'WeekTime':
    result = WeekTime(0,0,0)
    result.setMinute(value & 0xff)

    value = value >> 8
    result.setHour(value & 0x1F)
    result.setDay(value >> 5)
    return result

  def getDay(self) -> int:
    return self.day

  def setDay(self, day:int):
    self.day = day

  def getHour(self) -> int:
    return self.hour

  def setHour(self, hour:int):
    self.hour = hour

  def getMinute(self) -> int:
    return self.minute

  def setMinute(self, minute:int):
    self.minute = minute

  def __str__(self):
    return f"WeekTime(day={self.day}, hour={self.hour}, minute={self.minute})"


import threading
from pyhausbus.de.hausbus.homeassistant.proxy import ProxyFactory
from pyhausbus.HausBusUtils import *
from pyhausbus.BusDataMessage import BusDataMessage
from pyhausbus.IBusDataListener import IBusDataListener
import time
import importlib
from pyhausbus.UdpReceiveWorker import UdpReceiveWorker
import socket
import pyhausbus.HausBusUtils as HausBusUtils
import traceback

RS485_GATEWAY = "#RS485#"
EVENTS_START = 200
RESULT_START = 128

class BusHandler:

  _singleInstance = None
  sock:None
  broadcastIp = "192.168.178.255"
  listeners = []

  @staticmethod
  def getInstance():
    if BusHandler._singleInstance is None:
      BusHandler._singleInstance = BusHandler()
    return BusHandler._singleInstance

  def __init__(self):
    if BusHandler._singleInstance is None:
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
      self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
      x = UdpReceiveWorker(self.busDataReceived)
      x.startWorker()
      self._getBroadcastIp()

  def _getBroadcastIp(self):
    temp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        temp_socket.connect(("8.8.8.8", 80))
        own_ip = temp_socket.getsockname()[0].split('.')
        self.broadcastIp = own_ip[0] + "." + own_ip[1] + "." + own_ip[2] + ".255"
    finally:
        temp_socket.close()

  def busDataReceived(self, senderObjectId, receiverObjectId, functionId, functionData, gateway, corrupted:bool):
    # Es kann entweder eine Antwort oder Event des Senders sein oder ein Aufruf auf dem Empfänger
    featureClassId = 0
    identifierId = 0
    if (functionId < RESULT_START):
      featureClassId = getClassId(receiverObjectId)
      identifierId = receiverObjectId
    else:
      featureClassId = getClassId(senderObjectId)
      identifierId = senderObjectId

    LOGGER.debug("classId = " + str(featureClassId) + ", functionId = " + str(functionId))
    className = ProxyFactory.getBusClassNameFor(featureClassId, functionId)
    if (className=="de.hausbus.proxy.LogicalButton"):
      LOGGER.debug("test")

    try:
      module_name, class_name = className.rsplit(".", 1)
      module = importlib.import_module(className)
      cls = getattr(module, class_name)
      method = getattr(cls, "_fromBytes")
      offset = [0]
      newObject = method(functionData, offset)

      add = ""
      if (corrupted):
        add = " (corrupted) "

      message = gateway + " COMMAND IN " + add + " from " + str(getDeviceId(senderObjectId)) + " to " + str(getDeviceId(receiverObjectId)) + ": " + str(newObject) + ", Sender: " + formatObjectId(senderObjectId) + ", Receiver: " + str(formatObjectId(receiverObjectId))
      LOGGER.debug(message)

      if (not corrupted):
        newMessage = BusDataMessage(senderObjectId, receiverObjectId, newObject)
        LOGGER.debug("got: " + str(newObject) + " from " + str(senderObjectId) + " to " + str(receiverObjectId))
        for actListener in self.listeners:
          actListener.busDataReceived(newMessage)
    except (Exception, RuntimeError, TypeError, NameError, OSError) as err:
        LOGGER.error(err,exc_info=True,stack_info=True)

  def sendData(self, data:bytearray, debug:str):

    udpData:bytearray = self.prepareForUDP(data)

    LOGGER.debug(UdpReceiveWorker.UDP_GATEWAY + " COMMAND OUT " + debug)
    LOGGER.debug(UdpReceiveWorker.UDP_GATEWAY + " DATA OUT " + HausBusUtils.formatBytes(udpData))

    try:
      self.sock.sendto(udpData, (self.broadcastIp, UDP_PORT))
    except socket.error as e:
      LOGGER.error(e,exc_info=True,stack_info=True)

  def prepareForUDP(self, data:bytearray) -> bytearray:
    result = bytearray(len(data) + 2)
    result[0] = 0xef
    result[1] = 0xef
    result[2:] = data[:]
    return result

  def addBusEventListener(self, listener:IBusDataListener):
    if not listener in self.listeners:
      self.listeners.append(listener)

  def removeBusEventListener(self, listener:IBusDataListener):
    self.listeners.remove(listener)



import pyhausbus.HausBusUtils as HausBusUtils

class EvZeroCrossData:
  CLASS_ID = 0
  FUNCTION_ID = 209

  def __init__(self,channel:int, lowTime0:int, lowTime1:int, lowTime2:int, lowTime3:int, lowTime4:int, lowTime5:int, lowTime6:int, lowTime7:int, lowTime8:int, lowTime9:int, lowTime10:int, lowTime11:int, lowTime12:int, lowTime13:int, lowTime14:int, lowTime15:int, pulsWidth0:int, pulsWidth1:int, pulsWidth2:int, pulsWidth3:int, pulsWidth4:int, pulsWidth5:int, pulsWidth6:int, pulsWidth7:int, pulsWidth8:int, pulsWidth9:int, pulsWidth10:int, pulsWidth11:int, pulsWidth12:int, pulsWidth13:int, pulsWidth14:int, pulsWidth15:int):
    self.channel=channel
    self.lowTime0=lowTime0
    self.lowTime1=lowTime1
    self.lowTime2=lowTime2
    self.lowTime3=lowTime3
    self.lowTime4=lowTime4
    self.lowTime5=lowTime5
    self.lowTime6=lowTime6
    self.lowTime7=lowTime7
    self.lowTime8=lowTime8
    self.lowTime9=lowTime9
    self.lowTime10=lowTime10
    self.lowTime11=lowTime11
    self.lowTime12=lowTime12
    self.lowTime13=lowTime13
    self.lowTime14=lowTime14
    self.lowTime15=lowTime15
    self.pulsWidth0=pulsWidth0
    self.pulsWidth1=pulsWidth1
    self.pulsWidth2=pulsWidth2
    self.pulsWidth3=pulsWidth3
    self.pulsWidth4=pulsWidth4
    self.pulsWidth5=pulsWidth5
    self.pulsWidth6=pulsWidth6
    self.pulsWidth7=pulsWidth7
    self.pulsWidth8=pulsWidth8
    self.pulsWidth9=pulsWidth9
    self.pulsWidth10=pulsWidth10
    self.pulsWidth11=pulsWidth11
    self.pulsWidth12=pulsWidth12
    self.pulsWidth13=pulsWidth13
    self.pulsWidth14=pulsWidth14
    self.pulsWidth15=pulsWidth15


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvZeroCrossData(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvZeroCrossData(channel={self.channel}, lowTime0={self.lowTime0}, lowTime1={self.lowTime1}, lowTime2={self.lowTime2}, lowTime3={self.lowTime3}, lowTime4={self.lowTime4}, lowTime5={self.lowTime5}, lowTime6={self.lowTime6}, lowTime7={self.lowTime7}, lowTime8={self.lowTime8}, lowTime9={self.lowTime9}, lowTime10={self.lowTime10}, lowTime11={self.lowTime11}, lowTime12={self.lowTime12}, lowTime13={self.lowTime13}, lowTime14={self.lowTime14}, lowTime15={self.lowTime15}, pulsWidth0={self.pulsWidth0}, pulsWidth1={self.pulsWidth1}, pulsWidth2={self.pulsWidth2}, pulsWidth3={self.pulsWidth3}, pulsWidth4={self.pulsWidth4}, pulsWidth5={self.pulsWidth5}, pulsWidth6={self.pulsWidth6}, pulsWidth7={self.pulsWidth7}, pulsWidth8={self.pulsWidth8}, pulsWidth9={self.pulsWidth9}, pulsWidth10={self.pulsWidth10}, pulsWidth11={self.pulsWidth11}, pulsWidth12={self.pulsWidth12}, pulsWidth13={self.pulsWidth13}, pulsWidth14={self.pulsWidth14}, pulsWidth15={self.pulsWidth15})"

  '''
  @param channel PWM-Kanal.
  '''
  def getChannel(self):
    return self.channel

  '''
  @param lowTime0 .
  '''
  def getLowTime0(self):
    return self.lowTime0

  '''
  @param lowTime1 .
  '''
  def getLowTime1(self):
    return self.lowTime1

  '''
  @param lowTime2 .
  '''
  def getLowTime2(self):
    return self.lowTime2

  '''
  @param lowTime3 .
  '''
  def getLowTime3(self):
    return self.lowTime3

  '''
  @param lowTime4 .
  '''
  def getLowTime4(self):
    return self.lowTime4

  '''
  @param lowTime5 .
  '''
  def getLowTime5(self):
    return self.lowTime5

  '''
  @param lowTime6 .
  '''
  def getLowTime6(self):
    return self.lowTime6

  '''
  @param lowTime7 .
  '''
  def getLowTime7(self):
    return self.lowTime7

  '''
  @param lowTime8 .
  '''
  def getLowTime8(self):
    return self.lowTime8

  '''
  @param lowTime9 .
  '''
  def getLowTime9(self):
    return self.lowTime9

  '''
  @param lowTime10 .
  '''
  def getLowTime10(self):
    return self.lowTime10

  '''
  @param lowTime11 .
  '''
  def getLowTime11(self):
    return self.lowTime11

  '''
  @param lowTime12 .
  '''
  def getLowTime12(self):
    return self.lowTime12

  '''
  @param lowTime13 .
  '''
  def getLowTime13(self):
    return self.lowTime13

  '''
  @param lowTime14 .
  '''
  def getLowTime14(self):
    return self.lowTime14

  '''
  @param lowTime15 .
  '''
  def getLowTime15(self):
    return self.lowTime15

  '''
  @param pulsWidth0 .
  '''
  def getPulsWidth0(self):
    return self.pulsWidth0

  '''
  @param pulsWidth1 .
  '''
  def getPulsWidth1(self):
    return self.pulsWidth1

  '''
  @param pulsWidth2 .
  '''
  def getPulsWidth2(self):
    return self.pulsWidth2

  '''
  @param pulsWidth3 .
  '''
  def getPulsWidth3(self):
    return self.pulsWidth3

  '''
  @param pulsWidth4 .
  '''
  def getPulsWidth4(self):
    return self.pulsWidth4

  '''
  @param pulsWidth5 .
  '''
  def getPulsWidth5(self):
    return self.pulsWidth5

  '''
  @param pulsWidth6 .
  '''
  def getPulsWidth6(self):
    return self.pulsWidth6

  '''
  @param pulsWidth7 .
  '''
  def getPulsWidth7(self):
    return self.pulsWidth7

  '''
  @param pulsWidth8 .
  '''
  def getPulsWidth8(self):
    return self.pulsWidth8

  '''
  @param pulsWidth9 .
  '''
  def getPulsWidth9(self):
    return self.pulsWidth9

  '''
  @param pulsWidth10 .
  '''
  def getPulsWidth10(self):
    return self.pulsWidth10

  '''
  @param pulsWidth11 .
  '''
  def getPulsWidth11(self):
    return self.pulsWidth11

  '''
  @param pulsWidth12 .
  '''
  def getPulsWidth12(self):
    return self.pulsWidth12

  '''
  @param pulsWidth13 .
  '''
  def getPulsWidth13(self):
    return self.pulsWidth13

  '''
  @param pulsWidth14 .
  '''
  def getPulsWidth14(self):
    return self.pulsWidth14

  '''
  @param pulsWidth15 .
  '''
  def getPulsWidth15(self):
    return self.pulsWidth15






from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.params.MConfig import MConfig
from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Current import Current
from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.SignalCount import SignalCount
from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Power import Power

class CurrentReader(ABusFeature):
  CLASS_ID:int = 90

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return CurrentReader(HausBusUtils.getObjectId(deviceId, 90, instanceId))

  """
  @param config .
  @param impPerKwh Anzahl Signale pro kWh.
  @param startCurrent Startwert Stromverbrauch in Wattstunden.
  @param currentReportInterval Interval in Sekunden nach dem immer der aktuelle Gesamtstromverbrauch gemeldet wird.
  """
  def setConfiguration(self, config:MConfig, impPerKwh:int, startCurrent:int, currentReportInterval:int):
    LOGGER.debug("setConfiguration"+" config = "+str(config)+" impPerKwh = "+str(impPerKwh)+" startCurrent = "+str(startCurrent)+" currentReportInterval = "+str(currentReportInterval))
    hbCommand = HausBusCommand(self.objectId, 3, "setConfiguration")
    hbCommand.addByte(config.getValue())
    hbCommand.addWord(impPerKwh)
    hbCommand.addDWord(startCurrent)
    hbCommand.addWord(currentReportInterval)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 4, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param config .
  @param impPerKwh Anzahl Signale pro kWh.
  @param startCurrent Startwert Stromverbrauch in Wattstunden.
  @param currentReportInterval Interval in Sekunden nach dem immer der aktuelle Gesamtstromverbrauch gemeldet wird.
  """
  def Configuration(self, config:MConfig, impPerKwh:int, startCurrent:int, currentReportInterval:int):
    LOGGER.debug("Configuration"+" config = "+str(config)+" impPerKwh = "+str(impPerKwh)+" startCurrent = "+str(startCurrent)+" currentReportInterval = "+str(currentReportInterval))
    hbCommand = HausBusCommand(self.objectId, 129, "Configuration")
    hbCommand.addByte(config.getValue())
    hbCommand.addWord(impPerKwh)
    hbCommand.addDWord(startCurrent)
    hbCommand.addWord(currentReportInterval)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param time Systemzeit des ESP zu Debugzwecken.
  @param signalCount Anzahl gez?hlter S0 Signale seit dem letzten Zur?cksetzen.
  @param power Aktuelle Leistung in Watt.
  @param signalDuration Dauer des gemessenen S0 Signals in ms.
  """
  def evSignal(self, time:int, signalCount:int, power:int, signalDuration:int):
    LOGGER.debug("evSignal"+" time = "+str(time)+" signalCount = "+str(signalCount)+" power = "+str(power)+" signalDuration = "+str(signalDuration))
    hbCommand = HausBusCommand(self.objectId, 200, "evSignal")
    hbCommand.addDWord(time)
    hbCommand.addDWord(signalCount)
    hbCommand.addWord(power)
    hbCommand.addDWord(signalDuration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getCurrent(self):
    LOGGER.debug("getCurrent")
    hbCommand = HausBusCommand(self.objectId, 1, "getCurrent")
    ResultWorker()._setResultInfo(Current,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param current Verbrauchter Strom in Wattstunden.
  """
  def evCurrent(self, current:int):
    LOGGER.debug("evCurrent"+" current = "+str(current))
    hbCommand = HausBusCommand(self.objectId, 201, "evCurrent")
    hbCommand.addDWord(current)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param power Aktuelle Leistung in Watt.
  """
  def Power(self, power:int):
    LOGGER.debug("Power"+" power = "+str(power))
    hbCommand = HausBusCommand(self.objectId, 130, "Power")
    hbCommand.addWord(power)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param current verbrauchter Strom in Wattstunden.
  """
  def Current(self, current:int):
    LOGGER.debug("Current"+" current = "+str(current))
    hbCommand = HausBusCommand(self.objectId, 128, "Current")
    hbCommand.addDWord(current)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getSignalCount(self):
    LOGGER.debug("getSignalCount")
    hbCommand = HausBusCommand(self.objectId, 6, "getSignalCount")
    ResultWorker()._setResultInfo(SignalCount,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param signalCount Anzahl gez?hlter S0 Signale seit dem letzten Zur?cksetzen.
  """
  def SignalCount(self, signalCount:int):
    LOGGER.debug("SignalCount"+" signalCount = "+str(signalCount))
    hbCommand = HausBusCommand(self.objectId, 131, "SignalCount")
    hbCommand.addDWord(signalCount)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def clearSignalCount(self):
    LOGGER.debug("clearSignalCount")
    hbCommand = HausBusCommand(self.objectId, 7, "clearSignalCount")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param signalCount .
  """
  def setSignalCount(self, signalCount:int):
    LOGGER.debug("setSignalCount"+" signalCount = "+str(signalCount))
    hbCommand = HausBusCommand(self.objectId, 2, "setSignalCount")
    hbCommand.addDWord(signalCount)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getPower(self):
    LOGGER.debug("getPower")
    hbCommand = HausBusCommand(self.objectId, 5, "getPower")
    ResultWorker()._setResultInfo(Power,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def incSignalCount(self):
    LOGGER.debug("incSignalCount")
    hbCommand = HausBusCommand(self.objectId, 9, "incSignalCount")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def decSignalCount(self):
    LOGGER.debug("decSignalCount")
    hbCommand = HausBusCommand(self.objectId, 10, "decSignalCount")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param data .
  @param type .
  """
  def evDebug(self, data:int, type:MConfig):
    LOGGER.debug("evDebug"+" data = "+str(data)+" type = "+str(type))
    hbCommand = HausBusCommand(self.objectId, 210, "evDebug")
    hbCommand.addDWord(data)
    hbCommand.addByte(type.getValue())
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param value .
  @param stamp .
  """
  def evInterrupt(self, value:int, stamp:int):
    LOGGER.debug("evInterrupt"+" value = "+str(value)+" stamp = "+str(stamp))
    hbCommand = HausBusCommand(self.objectId, 211, "evInterrupt")
    hbCommand.addByte(value)
    hbCommand.addDWord(stamp)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.de.hausbus.homeassistant.proxy.currentReader.params.MConfig import MConfig
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 90
  FUNCTION_ID = 129

  def __init__(self,config:MConfig, impPerKwh:int, startCurrent:int, currentReportInterval:int):
    self.config=config
    self.impPerKwh=impPerKwh
    self.startCurrent=startCurrent
    self.currentReportInterval=currentReportInterval


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(MConfig._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"Configuration(config={self.config}, impPerKwh={self.impPerKwh}, startCurrent={self.startCurrent}, currentReportInterval={self.currentReportInterval})"

  '''
  @param config .
  '''
  def getConfig(self) -> MConfig:
    return self.config

  '''
  @param impPerKwh Anzahl Signale pro kWh.
  '''
  def getImpPerKwh(self):
    return self.impPerKwh

  '''
  @param startCurrent Startwert Stromverbrauch in Wattstunden.
  '''
  def getStartCurrent(self):
    return self.startCurrent

  '''
  @param currentReportInterval Interval in Sekunden nach dem immer der aktuelle Gesamtstromverbrauch gemeldet wird.
  '''
  def getCurrentReportInterval(self):
    return self.currentReportInterval






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EFeatureId(Enum):
  RFID=0
  PID=1
  MOD_BUS=2
  WIFI=3
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EFeatureId.__members__.values():
      if (act.value == checkValue):
        return act

    return EFeatureId.SER_UNKNOWN






from pyhausbus.de.hausbus.homeassistant.proxy.gateway.params.MOptions import MOptions
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 176
  FUNCTION_ID = 128

  def __init__(self,options:MOptions):
    self.options=options


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(MOptions._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Configuration(options={self.options})"

  '''
  @param options enabled: Dies Gateway ist aktiv und leitet Nachrichten weiter\r\npreferLoxone: Gateway kommuniziert bevorzugt im Loxone-Protokoll.
  '''
  def getOptions(self) -> MOptions:
    return self.options






from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.wiFi.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.wiFi.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.wiFi.data.CurrentIp import CurrentIp

class WiFi(ABusFeature):
  CLASS_ID:int = 164

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return WiFi(HausBusUtils.getObjectId(deviceId, 164, instanceId))

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param mac1 .
  @param mac2 .
  @param mac3 .
  @param mac4 .
  @param mac5 .
  @param mac6 .
  """
  def wakeUpDevice(self, mac1:int, mac2:int, mac3:int, mac4:int, mac5:int, mac6:int):
    LOGGER.debug("wakeUpDevice"+" mac1 = "+str(mac1)+" mac2 = "+str(mac2)+" mac3 = "+str(mac3)+" mac4 = "+str(mac4)+" mac5 = "+str(mac5)+" mac6 = "+str(mac6))
    hbCommand = HausBusCommand(self.objectId, 2, "wakeUpDevice")
    hbCommand.addByte(mac1)
    hbCommand.addByte(mac2)
    hbCommand.addByte(mac3)
    hbCommand.addByte(mac4)
    hbCommand.addByte(mac5)
    hbCommand.addByte(mac6)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param SSID .
  @param Password .
  @param Server_Port Zusaetzlicher Port fuer die Homeserverfunktionen z.B 15557 f?r Loxone oder 5855 f?r IOBroker.
  @param Server_IP0 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP1 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP2 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP3 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  """
  def setConfiguration(self, SSID:str, Password:str, Server_Port:int, Server_IP0:int, Server_IP1:int, Server_IP2:int, Server_IP3:int):
    LOGGER.debug("setConfiguration"+" SSID = "+str(SSID)+" Password = "+str(Password)+" Server_Port = "+str(Server_Port)+" Server_IP0 = "+str(Server_IP0)+" Server_IP1 = "+str(Server_IP1)+" Server_IP2 = "+str(Server_IP2)+" Server_IP3 = "+str(Server_IP3))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addString(SSID)
    hbCommand.addString(Password)
    hbCommand.addWord(Server_Port)
    hbCommand.addByte(Server_IP0)
    hbCommand.addByte(Server_IP1)
    hbCommand.addByte(Server_IP2)
    hbCommand.addByte(Server_IP3)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param SSID .
  @param Password .
  @param Server_Port Zusaetzlicher Port fuer die Homeserverfunktionen z.B 15557 f?r Loxone oder 5855 f?r IOBroker.
  @param Server_IP0 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP1 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP2 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  @param Server_IP3 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  """
  def Configuration(self, SSID:str, Password:str, Server_Port:int, Server_IP0:int, Server_IP1:int, Server_IP2:int, Server_IP3:int):
    LOGGER.debug("Configuration"+" SSID = "+str(SSID)+" Password = "+str(Password)+" Server_Port = "+str(Server_Port)+" Server_IP0 = "+str(Server_IP0)+" Server_IP1 = "+str(Server_IP1)+" Server_IP2 = "+str(Server_IP2)+" Server_IP3 = "+str(Server_IP3))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addString(SSID)
    hbCommand.addString(Password)
    hbCommand.addWord(Server_Port)
    hbCommand.addByte(Server_IP0)
    hbCommand.addByte(Server_IP1)
    hbCommand.addByte(Server_IP2)
    hbCommand.addByte(Server_IP3)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getCurrentIp(self):
    LOGGER.debug("getCurrentIp")
    hbCommand = HausBusCommand(self.objectId, 3, "getCurrentIp")
    ResultWorker()._setResultInfo(CurrentIp,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param IP0 .
  @param IP1 .
  @param IP2 .
  @param IP3 .
  """
  def CurrentIp(self, IP0:int, IP1:int, IP2:int, IP3:int):
    LOGGER.debug("CurrentIp"+" IP0 = "+str(IP0)+" IP1 = "+str(IP1)+" IP2 = "+str(IP2)+" IP3 = "+str(IP3))
    hbCommand = HausBusCommand(self.objectId, 129, "CurrentIp")
    hbCommand.addByte(IP0)
    hbCommand.addByte(IP1)
    hbCommand.addByte(IP2)
    hbCommand.addByte(IP3)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.led.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.led.params.MOptions import MOptions
from pyhausbus.de.hausbus.homeassistant.proxy.led.data.Status import Status
from pyhausbus.de.hausbus.homeassistant.proxy.led.data.MinBrightness import MinBrightness

class Led(ABusFeature):
  CLASS_ID:int = 21

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return Led(HausBusUtils.getObjectId(deviceId, 21, instanceId))

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param dimmOffset 0-100% offset auf den im Kommando angegebenen Helligkeitswert.
  @param minBrightness Eine ausgeschaltete LED leuchtet immer noch mit dieser Helligkeit 0-100%.
  @param timeBase Zeitbasis [ms] fuer Zeitabhaengige Befehle..
  @param options Reservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  """
  def setConfiguration(self, dimmOffset:int, minBrightness:int, timeBase:int, options:MOptions):
    LOGGER.debug("setConfiguration"+" dimmOffset = "+str(dimmOffset)+" minBrightness = "+str(minBrightness)+" timeBase = "+str(timeBase)+" options = "+str(options))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(dimmOffset)
    hbCommand.addByte(minBrightness)
    hbCommand.addWord(timeBase)
    hbCommand.addByte(options.getValue())
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param brightness 0-100% Helligkeit.
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos.
  @param onDelay Einschaltverzoegerung: Wert * Zeitbasis [ms]\r\n0=Keine.
  """
  def on(self, brightness:int, duration:int, onDelay:int):
    LOGGER.debug("on"+" brightness = "+str(brightness)+" duration = "+str(duration)+" onDelay = "+str(onDelay))
    hbCommand = HausBusCommand(self.objectId, 3, "on")
    hbCommand.addByte(brightness)
    hbCommand.addWord(duration)
    hbCommand.addWord(onDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param brightness 0-100% Helligkeit.
  @param offTime Ausschaltdauer: \r\nWert * Zeitbasis [ms].
  @param onTime Einschaltdauer: \r\nWert * Zeitbasis [ms].
  @param quantity Anzahl Blinks.
  """
  def blink(self, brightness:int, offTime:int, onTime:int, quantity:int):
    LOGGER.debug("blink"+" brightness = "+str(brightness)+" offTime = "+str(offTime)+" onTime = "+str(onTime)+" quantity = "+str(quantity))
    hbCommand = HausBusCommand(self.objectId, 4, "blink")
    hbCommand.addByte(brightness)
    hbCommand.addByte(offTime)
    hbCommand.addByte(onTime)
    hbCommand.addByte(quantity)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getStatus(self):
    LOGGER.debug("getStatus")
    hbCommand = HausBusCommand(self.objectId, 5, "getStatus")
    ResultWorker()._setResultInfo(Status,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param dimmOffset 0-100% offset auf den im Kommando angegebenen Helligkeitswert.
  @param minBrightness Eine ausgeschaltete LED leuchtet immer noch mit dieser Helligkeit 0-100%.
  @param timeBase Zeitbasis [ms] f?  ? ? ?r Zeitabh?  ? ? ??ngige Befehle..
  @param options Reservierte Bits m?  ? ? ?ssen immer deaktiviert sein. Das Aktivieren eines reservierten Bits f?  ? ? ?hrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  """
  def Configuration(self, dimmOffset:int, minBrightness:int, timeBase:int, options:MOptions):
    LOGGER.debug("Configuration"+" dimmOffset = "+str(dimmOffset)+" minBrightness = "+str(minBrightness)+" timeBase = "+str(timeBase)+" options = "+str(options))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(dimmOffset)
    hbCommand.addByte(minBrightness)
    hbCommand.addWord(timeBase)
    hbCommand.addByte(options.getValue())
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param brightness Helligkeit der LED.
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos.
  """
  def Status(self, brightness:int, duration:int):
    LOGGER.debug("Status"+" brightness = "+str(brightness)+" duration = "+str(duration))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addByte(brightness)
    hbCommand.addWord(duration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOff(self):
    LOGGER.debug("evOff")
    hbCommand = HausBusCommand(self.objectId, 200, "evOff")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param brightness 0-100% Helligkeit.
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos.
  """
  def evOn(self, brightness:int, duration:int):
    LOGGER.debug("evOn"+" brightness = "+str(brightness)+" duration = "+str(duration))
    hbCommand = HausBusCommand(self.objectId, 201, "evOn")
    hbCommand.addByte(brightness)
    hbCommand.addWord(duration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evBlink(self):
    LOGGER.debug("evBlink")
    hbCommand = HausBusCommand(self.objectId, 202, "evBlink")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:int):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param offDelay Ausschaltverzoegerung: Wert * Zeitbasis [ms]\r\n0=Keine.
  """
  def off(self, offDelay:int):
    LOGGER.debug("off"+" offDelay = "+str(offDelay))
    hbCommand = HausBusCommand(self.objectId, 2, "off")
    hbCommand.addWord(offDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param minBrightness Eine ausgeschaltete LED leuchtet immer noch mit dieser Helligkeit 0-100%.
  """
  def setMinBrightness(self, minBrightness:int):
    LOGGER.debug("setMinBrightness"+" minBrightness = "+str(minBrightness))
    hbCommand = HausBusCommand(self.objectId, 6, "setMinBrightness")
    hbCommand.addByte(minBrightness)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param cmdDelay Dauer Wert * Zeitbasis [ms].
  """
  def evCmdDelay(self, cmdDelay:int):
    LOGGER.debug("evCmdDelay"+" cmdDelay = "+str(cmdDelay))
    hbCommand = HausBusCommand(self.objectId, 203, "evCmdDelay")
    hbCommand.addWord(cmdDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getMinBrightness(self):
    LOGGER.debug("getMinBrightness")
    hbCommand = HausBusCommand(self.objectId, 7, "getMinBrightness")
    ResultWorker()._setResultInfo(MinBrightness,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param minBrightness Eine ausgeschaltete LED leuchtet immer noch mit dieser Helligkeit 0-100%.
  """
  def MinBrightness(self, minBrightness:int):
    LOGGER.debug("MinBrightness"+" minBrightness = "+str(minBrightness))
    hbCommand = HausBusCommand(self.objectId, 130, "MinBrightness")
    hbCommand.addByte(minBrightness)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EState(Enum):
  STARTUP=0
  STANDBY=1
  IDLE=2
  CONNECTED=3
  DISCONNECTED=4
  UNKNOWN=255
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EState.__members__.values():
      if (act.value == checkValue):
        return act

    return EState.SER_UNKNOWN






import pyhausbus.HausBusUtils as HausBusUtils

class EvDay:
  CLASS_ID = 0
  FUNCTION_ID = 206

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvDay()

  def __str__(self):
    return f"EvDay()"






import pyhausbus.HausBusUtils as HausBusUtils

class EvHot:
  CLASS_ID = 32
  FUNCTION_ID = 202

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvHot()

  def __str__(self):
    return f"EvHot()"






def getBusClassNameFor(classId, functionId):
  if (classId==16 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvClicked"
  if (classId==16 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvDoubleClick"
  if (classId==16 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvHoldStart"
  if (classId==16 and functionId==204):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvHoldEnd"
  if (classId==16 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.commands.GetConfiguration"
  if (classId==16 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.commands.SetConfiguration"
  if (classId==16 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Configuration"
  if (classId==16 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvCovered"
  if (classId==16 and functionId==205):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvFree"
  if (classId==16 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvError"
  if (classId==16 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.commands.EnableEvents"
  if (classId==16 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.commands.GetStatus"
  if (classId==16 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Status"
  if (classId==16 and functionId==206):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.EvEnabled"
  if (classId==16 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Enabled"
  if (classId==16 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taster.commands.GetEnabled"
  if (classId==32 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.EvCold"
  if (classId==32 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.EvWarm"
  if (classId==32 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.EvHot"
  if (classId==32 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.commands.GetConfiguration"
  if (classId==32 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.Configuration"
  if (classId==32 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.commands.SetConfiguration"
  if (classId==32 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.commands.GetStatus"
  if (classId==32 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.EvStatus"
  if (classId==32 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.EvError"
  if (classId==32 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.temperatursensor.data.Status"
  if (classId==19 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.Toggle"
  if (classId==19 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.On"
  if (classId==19 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.Off"
  if (classId==19 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvOn"
  if (classId==19 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvOff"
  if (classId==19 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.GetStatus"
  if (classId==19 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.Status"
  if (classId==19 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvError"
  if (classId==19 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvToggle"
  if (classId==19 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.Configuration"
  if (classId==19 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.GetConfiguration"
  if (classId==19 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.commands.SetConfiguration"
  if (classId==19 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvCmdDelay"
  if (classId==19 and functionId==204):
    return "pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.EvDisabled"
  if (classId==17 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvOn"
  if (classId==17 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.SetConfiguration"
  if (classId==17 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.GetConfiguration"
  if (classId==17 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.SetBrightness"
  if (classId==17 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.Configuration"
  if (classId==17 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvOff"
  if (classId==17 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvStart"
  if (classId==17 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.Start"
  if (classId==17 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.Stop"
  if (classId==17 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.commands.GetStatus"
  if (classId==17 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.Status"
  if (classId==17 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvError"
  if (classId==0 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetModuleId"
  if (classId==0 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetConfiguration"
  if (classId==0 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.ModuleId"
  if (classId==0 and functionId==131):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.Configuration"
  if (classId==0 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetRemoteObjects"
  if (classId==0 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.RemoteObjects"
  if (classId==0 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GenerateRandomDeviceId"
  if (classId==0 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.Reset"
  if (classId==0 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetUnusedMemory"
  if (classId==0 and functionId==6):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetConfiguration"
  if (classId==0 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.UnusedMemory"
  if (classId==0 and functionId==7):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.ReadMemory"
  if (classId==0 and functionId==8):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.WriteMemory"
  if (classId==0 and functionId==127):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.Ping"
  if (classId==0 and functionId==199):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.Pong"
  if (classId==0 and functionId==132):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.MemoryData"
  if (classId==0 and functionId==133):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.MemoryStatus"
  if (classId==0 and functionId==9):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.WriteRules"
  if (classId==0 and functionId==10):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.ReadRules"
  if (classId==0 and functionId==134):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.RulesData"
  if (classId==0 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvTime"
  if (classId==0 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvNewDeviceId"
  if (classId==0 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvStarted"
  if (classId==0 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvError"
  if (classId==0 and functionId==126):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetTime"
  if (classId==0 and functionId==125):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetTime"
  if (classId==0 and functionId==198):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.Time"
  if (classId==0 and functionId==12):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetRuleState"
  if (classId==0 and functionId==11):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetRuleState"
  if (classId==0 and functionId==13):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.TriggerRuleElement"
  if (classId==0 and functionId==135):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.RuleState"
  if (classId==0 and functionId==14):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetUnitGroupState"
  if (classId==0 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvGroupOn"
  if (classId==0 and functionId==205):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvGroupOff"
  if (classId==0 and functionId==204):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvGroupUndefined"
  if (classId==0 and functionId==136):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.TriggeredRule"
  if (classId==0 and functionId==124):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetDebugOptions"
  if (classId==0 and functionId==197):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.TimeD  ifference"
  if (classId==0 and functionId==206):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvDay"
  if (classId==0 and functionId==207):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvNight"
  if (classId==0 and functionId==15):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetSunTimes"
  if (classId==0 and functionId==16):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.SetSystemVariable"
  if (classId==0 and functionId==17):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetSystemVariable"
  if (classId==0 and functionId==137):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.SystemVariable"
  if (classId==0 and functionId==18):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.GetUnitGroupStatus"
  if (classId==0 and functionId==138):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.UnitGroupStatus"
  if (classId==0 and functionId==250):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvConsole"
  if (classId==0 and functionId==208):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvResetW  ifi"
  if (classId==0 and functionId==209):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvZeroCrossData"
  if (classId==0 and functionId==19):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.commands.EnableFeature"
  if (classId==0 and functionId==210):
    return "pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvSystemVariableChanged"
  if (classId==18 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.GetConfiguration"
  if (classId==18 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.MoveToPosition"
  if (classId==18 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.Start"
  if (classId==18 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.Stop"
  if (classId==18 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.Configuration"
  if (classId==18 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvClosed"
  if (classId==18 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvStart"
  if (classId==18 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.GetStatus"
  if (classId==18 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.Status"
  if (classId==18 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.SetConfiguration"
  if (classId==18 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvError"
  if (classId==18 and functionId==6):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.commands.SetPosition"
  if (classId==18 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvOpen"
  if (classId==18 and functionId==251):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvNewMainState"
  if (classId==18 and functionId==252):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rollladen.data.EvNewSubState"
  if (classId==15 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.digitalPort.commands.GetConfiguration"
  if (classId==15 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.digitalPort.commands.SetConfiguration"
  if (classId==15 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.digitalPort.data.Configuration"
  if (classId==15 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.digitalPort.data.EvError"
  if (classId==20 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.GetConfiguration"
  if (classId==20 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.SetConfiguration"
  if (classId==20 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.Configuration"
  if (classId==20 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.EvError"
  if (classId==20 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.EvOff"
  if (classId==20 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.EvOn"
  if (classId==20 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.EvBlink"
  if (classId==20 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.Off"
  if (classId==20 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.On"
  if (classId==20 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.Blink"
  if (classId==20 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.data.Status"
  if (classId==20 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.GetStatus"
  if (classId==20 and functionId==11):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.SetButtonConfiguration"
  if (classId==20 and functionId==12):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.SetLedConfiguration"
  if (classId==20 and functionId==6):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.SetMinBrightness"
  if (classId==20 and functionId==7):
    return "pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.commands.GetMinBrightness"
  if (classId==21 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.GetConfiguration"
  if (classId==21 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.SetConfiguration"
  if (classId==21 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.On"
  if (classId==21 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.Blink"
  if (classId==21 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.GetStatus"
  if (classId==21 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.Configuration"
  if (classId==21 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.Status"
  if (classId==21 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.EvOff"
  if (classId==21 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.EvOn"
  if (classId==21 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.EvBlink"
  if (classId==21 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.EvError"
  if (classId==21 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.Off"
  if (classId==21 and functionId==6):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.SetMinBrightness"
  if (classId==21 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.EvCmdDelay"
  if (classId==21 and functionId==7):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.commands.GetMinBrightness"
  if (classId==21 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.led.data.MinBrightness"
  if (classId==33 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.commands.Off"
  if (classId==33 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.commands.On"
  if (classId==33 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.data.EvClicked"
  if (classId==33 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.data.EvOff"
  if (classId==33 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.data.EvOn"
  if (classId==33 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.data.EvHoldStart"
  if (classId==33 and functionId==204):
    return "pyhausbus.de.hausbus.homeassistant.proxy.iRSensor.data.EvHoldEnd"
  if (classId==160 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.commands.GetConfiguration"
  if (classId==160 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.commands.SetConfiguration"
  if (classId==160 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.commands.AllOff"
  if (classId==160 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.commands.AllOn"
  if (classId==160 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.commands.SendCommand"
  if (classId==160 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.data.Configuration"
  if (classId==160 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.data.Status"
  if (classId==160 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.daliLine.data.EvError"
  if (classId==162 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.commands.WakeUpDevice"
  if (classId==162 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.data.Configuration"
  if (classId==162 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.commands.SetConfiguration"
  if (classId==162 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.commands.GetConfiguration"
  if (classId==162 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.data.EvError"
  if (classId==162 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.data.CurrentIp"
  if (classId==162 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ethernet.commands.GetCurrentIp"
  if (classId==34 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.EvDry"
  if (classId==34 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.EvConfortable"
  if (classId==34 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.EvWet"
  if (classId==34 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.EvError"
  if (classId==34 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.commands.GetConfiguration"
  if (classId==34 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.commands.SetConfiguration"
  if (classId==34 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.commands.GetStatus"
  if (classId==34 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.Configuration"
  if (classId==34 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.EvStatus"
  if (classId==34 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.feuchtesensor.data.Status"
  if (classId==1 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.Exec"
  if (classId==1 and functionId==126):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.SetVariable"
  if (classId==1 and functionId==11):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.Shutdown"
  if (classId==1 and functionId==12):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.Restart"
  if (classId==1 and functionId==20):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.Quit"
  if (classId==1 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.data.EvOnline"
  if (classId==1 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.data.EvOffline"
  if (classId==1 and functionId==10):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.Standby"
  if (classId==1 and functionId==13):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pCServer.commands.ReloadUserPlugin"
  if (classId==2 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wetter.commands.GetWeather"
  if (classId==2 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wetter.data.Weather"
  if (classId==176 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.EvError"
  if (classId==176 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.GetConfiguration"
  if (classId==176 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.SetConfiguration"
  if (classId==176 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.CheckBusTiming"
  if (classId==176 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.GetBusTiming"
  if (classId==176 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.BusTiming"
  if (classId==176 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.Configuration"
  if (classId==176 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.ResetBusTiming"
  if (classId==176 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.commands.GetConnectedDevices"
  if (classId==176 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.ConnectedDevices"
  if (classId==176 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.gateway.data.EvGatewayLoad"
  if (classId==35 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.commands.GetConfiguration"
  if (classId==35 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.commands.SetConfiguration"
  if (classId==35 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.commands.GetStatus"
  if (classId==35 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.commands.SetCount"
  if (classId==35 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.data.Configuration"
  if (classId==35 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.data.EvError"
  if (classId==35 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.data.Status"
  if (classId==35 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.counter.data.EvStatus"
  if (classId==3 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.systemTime.commands.GetTime"
  if (classId==3 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.systemTime.commands.SetTime"
  if (classId==3 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.systemTime.data.Time"
  if (classId==3 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.systemTime.data.EvError"
  if (classId==90 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.SetConfiguration"
  if (classId==90 and functionId==4):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.GetConfiguration"
  if (classId==90 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Configuration"
  if (classId==90 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.EvSignal"
  if (classId==90 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.GetCurrent"
  if (classId==90 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.EvCurrent"
  if (classId==90 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Power"
  if (classId==90 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.Current"
  if (classId==90 and functionId==6):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.GetSignalCount"
  if (classId==90 and functionId==131):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.SignalCount"
  if (classId==90 and functionId==7):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.ClearSignalCount"
  if (classId==90 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.SetSignalCount"
  if (classId==90 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.GetPower"
  if (classId==90 and functionId==9):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.IncSignalCount"
  if (classId==90 and functionId==10):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.commands.DecSignalCount"
  if (classId==90 and functionId==210):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.EvDebug"
  if (classId==90 and functionId==211):
    return "pyhausbus.de.hausbus.homeassistant.proxy.currentReader.data.EvInterrupt"
  if (classId==91 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.tcpClient.commands.AnnounceServer"
  if (classId==91 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.tcpClient.commands.GetCurrentIp"
  if (classId==91 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.tcpClient.data.CurrentIp"
  if (classId==91 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.tcpClient.data.EvWhoIsServer"
  if (classId==164 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.data.EvError"
  if (classId==164 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.commands.GetConfiguration"
  if (classId==164 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.commands.WakeUpDevice"
  if (classId==164 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.commands.SetConfiguration"
  if (classId==164 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.data.Configuration"
  if (classId==164 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.commands.GetCurrentIp"
  if (classId==164 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.wiFi.data.CurrentIp"
  if (classId==39 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.EvStatus"
  if (classId==39 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.EvDark"
  if (classId==39 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.EvLight"
  if (classId==39 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.EvBright"
  if (classId==39 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.commands.GetConfiguration"
  if (classId==39 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.commands.SetConfiguration"
  if (classId==39 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.commands.GetStatus"
  if (classId==39 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.Configuration"
  if (classId==39 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.EvError"
  if (classId==39 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.data.Status"
  if (classId==41 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.EvLowPower"
  if (classId==41 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.EvMediumPower"
  if (classId==41 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.EvHighPower"
  if (classId==41 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.EvError"
  if (classId==41 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.commands.GetConfiguration"
  if (classId==41 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.commands.SetConfiguration"
  if (classId==41 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.commands.GetStatus"
  if (classId==41 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.Configuration"
  if (classId==41 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.Status"
  if (classId==41 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.EvStatus"
  if (classId==22 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.data.EvOff"
  if (classId==22 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.data.EvOn"
  if (classId==22 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.commands.SetColor"
  if (classId==22 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.commands.GetConfiguration"
  if (classId==22 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.commands.SetConfiguration"
  if (classId==22 and functionId==5):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.commands.GetStatus"
  if (classId==22 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.data.Configuration"
  if (classId==22 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rGBDimmer.data.Status"
  if (classId==42 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.EvLow"
  if (classId==42 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.EvInRange"
  if (classId==42 and functionId==202):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.EvAbove"
  if (classId==42 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.EvError"
  if (classId==42 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.commands.GetConfiguration"
  if (classId==42 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.commands.SetConfiguration"
  if (classId==42 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.commands.GetStatus"
  if (classId==42 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.Configuration"
  if (classId==42 and functionId==203):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.EvStatus"
  if (classId==42 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.taupunkt.data.Status"
  if (classId==43 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.EvConnected"
  if (classId==43 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.EvError"
  if (classId==43 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.commands.GetConfiguration"
  if (classId==43 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.commands.SetConfiguration"
  if (classId==43 and functionId==129):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.State"
  if (classId==43 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.EvData"
  if (classId==43 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.commands.GetLastData"
  if (classId==43 and functionId==130):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.LastData"
  if (classId==43 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.Configuration"
  if (classId==43 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.commands.GetState"
  if (classId==44 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.commands.GetConfiguration"
  if (classId==44 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.commands.SetConfiguration"
  if (classId==44 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.data.Configuration"
  if (classId==44 and functionId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.commands.SetTargetValue"
  if (classId==44 and functionId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.commands.Enable"
  if (classId==44 and functionId==200):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.data.EvOn"
  if (classId==44 and functionId==255):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.data.EvError"
  if (classId==44 and functionId==201):
    return "pyhausbus.de.hausbus.homeassistant.proxy.pIDController.data.EvOff"
  if (classId==45 and functionId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.commands.GetConfiguration"
  if (classId==45 and functionId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.commands.SetConfiguration"
  if (classId==45 and functionId==128):
    return "pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.data.Configuration"

  return "de.hausbus.com.GenericBusData"

def getBusClassNameForClass(classId):
  if (classId==16):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Taster"
  if (classId==32):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Temperatursensor"
  if (classId==19):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Schalter"
  if (classId==17):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Dimmer"
  if (classId==0):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Controller"
  if (classId==18):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Rollladen"
  if (classId==15):
    return "pyhausbus.de.hausbus.homeassistant.proxy.DigitalPort"
  if (classId==20):
    return "pyhausbus.de.hausbus.homeassistant.proxy.LogicalButton"
  if (classId==21):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Led"
  if (classId==33):
    return "pyhausbus.de.hausbus.homeassistant.proxy.IRSensor"
  if (classId==160):
    return "pyhausbus.de.hausbus.homeassistant.proxy.DaliLine"
  if (classId==162):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Ethernet"
  if (classId==34):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Feuchtesensor"
  if (classId==1):
    return "pyhausbus.de.hausbus.homeassistant.proxy.PCServer"
  if (classId==2):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Wetter"
  if (classId==14):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ModbusSlave"
  if (classId==176):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Gateway"
  if (classId==35):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Counter"
  if (classId==163):
    return "pyhausbus.de.hausbus.homeassistant.proxy.SnmpAgent"
  if (classId==3):
    return "pyhausbus.de.hausbus.homeassistant.proxy.SystemTime"
  if (classId==90):
    return "pyhausbus.de.hausbus.homeassistant.proxy.CurrentReader"
  if (classId==91):
    return "pyhausbus.de.hausbus.homeassistant.proxy.TcpClient"
  if (classId==164):
    return "pyhausbus.de.hausbus.homeassistant.proxy.WiFi"
  if (classId==39):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Helligkeitssensor"
  if (classId==41):
    return "pyhausbus.de.hausbus.homeassistant.proxy.PowerMeter"
  if (classId==22):
    return "pyhausbus.de.hausbus.homeassistant.proxy.RGBDimmer"
  if (classId==42):
    return "pyhausbus.de.hausbus.homeassistant.proxy.Taupunkt"
  if (classId==43):
    return "pyhausbus.de.hausbus.homeassistant.proxy.RFIDReader"
  if (classId==44):
    return "pyhausbus.de.hausbus.homeassistant.proxy.PIDController"
  if (classId==45):
    return "pyhausbus.de.hausbus.homeassistant.proxy.ModBusMaster"

  return "de.hausbus.com.GenericBusData"




import pyhausbus.HausBusUtils as HausBusUtils
class MOptions:

  def setEnabled(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 0, self.value)
    return self

  def isEnabled(self):
    return HausBusUtils.isBitSet(0, self.value)
  def setPreferLoxone(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 1, self.value)
    return self

  def isPreferLoxone(self):
    return HausBusUtils.isBitSet(1, self.value)
  def setEnableConsole(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 2, self.value)
    return self

  def isEnableConsole(self):
    return HausBusUtils.isBitSet(2, self.value)
  def setReserved3(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 3, self.value)
    return self

  def isReserved3(self):
    return HausBusUtils.isBitSet(3, self.value)
  def setReserved4(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 4, self.value)
    return self

  def isReserved4(self):
    return HausBusUtils.isBitSet(4, self.value)
  def setReserved5(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 5, self.value)
    return self

  def isReserved5(self):
    return HausBusUtils.isBitSet(5, self.value)
  def setReserved6(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 6, self.value)
    return self

  def isReserved6(self):
    return HausBusUtils.isBitSet(6, self.value)
  def setReserved7(self, setValue:bool):
    self.value = HausBusUtils.setBit(setValue, 7, self.value)
    return self

  def isReserved7(self):
    return HausBusUtils.isBitSet(7, self.value)
  def __init__(self, value:int):
    self.value = value

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    return MOptions(HausBusUtils.bytesToInt(data, offset))



  def getValue(self):
    return self.value
  def getEntryNames(self):
    result = []
    result.append("Enabled")
    result.append("PreferLoxone")
    result.append("EnableConsole")
    result.append("Reserved3")
    result.append("Reserved4")
    result.append("Reserved5")
    result.append("Reserved6")
    result.append("Reserved7")
    return result
  def setEntry(self,name:str, setValue:bool):
    if (name == "Enabled"):
      self.setEnabled(setValue)
    if (name == "PreferLoxone"):
      self.setPreferLoxone(setValue)
    if (name == "EnableConsole"):
      self.setEnableConsole(setValue)
    if (name == "Reserved3"):
      self.setReserved3(setValue)
    if (name == "Reserved4"):
      self.setReserved4(setValue)
    if (name == "Reserved5"):
      self.setReserved5(setValue)
    if (name == "Reserved6"):
      self.setReserved6(setValue)
    if (name == "Reserved7"):
      self.setReserved7(setValue)

  def __str__(self):
    return f"MOptions(Enabled = {self.isEnabled()}, PreferLoxone = {self.isPreferLoxone()}, EnableConsole = {self.isEnableConsole()}, Reserved3 = {self.isReserved3()}, Reserved4 = {self.isReserved4()}, Reserved5 = {self.isReserved5()}, Reserved6 = {self.isReserved6()}, Reserved7 = {self.isReserved7()})"






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EErrorCode(Enum):
  NO_ERROR=0
  MODULE_NOT_EXISTS=1
  MEMORY_WRITE_FAILED=2
  INVALID_RULE_TABLE=3
  SYNTAX_ERROR=4
  NULL_POINTER=5
  LOW_VOLTAGE=6
  WATCHDOG=7
  INVALID_FW_LOADED=8
  MSG_QUEUE_OVERRUN=9
  CHECKSUM_ERROR=16
  CHECKSUM_ERROR_CONF_FLASH=17
  CHECKSUM_ERROR_CONF_EEPROM=18
  UNIT_GROUP_NOT_EXISTS=19
  MAX_OBJECTS_REACHED=10
  MODBUS_STREAM_NOT_EXISTS=20
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EErrorCode.__members__.values():
      if (act.value == checkValue):
        return act

    return EErrorCode.SER_UNKNOWN






import pyhausbus.HausBusUtils as HausBusUtils

class EvConsole:
  CLASS_ID = 0
  FUNCTION_ID = 250

  def __init__(self,consoleString:str):
    self.consoleString=consoleString


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvConsole(HausBusUtils.bytesToString(dataIn, offset))

  def __str__(self):
    return f"EvConsole(consoleString={self.consoleString})"

  '''
  @param consoleString Debug Ausgaben bei spezieller Firmware zur Fehlersuche.
  '''
  def getConsoleString(self):
    return self.consoleString






from pyhausbus.de.hausbus.homeassistant.proxy.schalter.params.MOptions import MOptions
import pyhausbus.HausBusUtils as HausBusUtils

class SetConfiguration:
  CLASS_ID = 19
  FUNCTION_ID = 1

  def __init__(self,maxOnTime:int, offDelayTime:int, timeBase:int, options:MOptions, disableBitIndex:int):
    self.maxOnTime=maxOnTime
    self.offDelayTime=offDelayTime
    self.timeBase=timeBase
    self.options=options
    self.disableBitIndex=disableBitIndex


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetConfiguration(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), MOptions._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"SetConfiguration(maxOnTime={self.maxOnTime}, offDelayTime={self.offDelayTime}, timeBase={self.timeBase}, options={self.options}, disableBitIndex={self.disableBitIndex})"

  '''
  @param maxOnTime Maximale Zeit.
  '''
  def getMaxOnTime(self):
    return self.maxOnTime

  '''
  @param offDelayTime Verzoegerungszeit nach einem Off-Kommando.
  '''
  def getOffDelayTime(self):
    return self.offDelayTime

  '''
  @param timeBase Zeitbasis [ms] fuer die Zeitabhaengigen Befehle.
  '''
  def getTimeBase(self):
    return self.timeBase

  '''
  @param options Reservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  '''
  def getOptions(self) -> MOptions:
    return self.options

  '''
  @param disableBitIndex Bit Index0-31 Systemvariable.
  '''
  def getDisableBitIndex(self):
    return self.disableBitIndex






import pyhausbus.HausBusUtils as HausBusUtils

class BusTiming:
  CLASS_ID = 176
  FUNCTION_ID = 129

  def __init__(self,timings):
    self.timings=timings


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return BusTiming(HausBusUtils.bytesToList(dataIn, offset))

  def __str__(self):
    return f"BusTiming(timings={self.timings})"

  '''
  @param timings .
  '''
  def getTimings(self):
    return self.timings






import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 44
  FUNCTION_ID = 128

  def __init__(self,P:int, I:int, D:int, targetValue:int, sensorObjectId:int, actorObjectId:int, timeout:int, hysteresis:int):
    self.P=P
    self.I=I
    self.D=D
    self.targetValue=targetValue
    self.sensorObjectId=sensorObjectId
    self.actorObjectId=actorObjectId
    self.timeout=timeout
    self.hysteresis=hysteresis


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"Configuration(P={self.P}, I={self.I}, D={self.D}, targetValue={self.targetValue}, sensorObjectId={self.sensorObjectId}, actorObjectId={self.actorObjectId}, timeout={self.timeout}, hysteresis={self.hysteresis})"

  '''
  @param P P-Anteil des Reglers.
  '''
  def getP(self):
    return self.P

  '''
  @param I I-Anteil des Reglers.
  '''
  def getI(self):
    return self.I

  '''
  @param D D-Anteil des Reglers.
  '''
  def getD(self):
    return self.D

  '''
  @param targetValue Regelungszielwert z.B. targetValue*0.
  '''
  def getTargetValue(self):
    return self.targetValue

  '''
  @param sensorObjectId Komplette Objekt-ID des Feedback-Sensors.
  '''
  def getSensorObjectId(self):
    return self.sensorObjectId

  '''
  @param actorObjectId Komplette Objekt-ID des Stellers.
  '''
  def getActorObjectId(self):
    return self.actorObjectId

  '''
  @param timeout Zeit.
  '''
  def getTimeout(self):
    return self.timeout

  '''
  @param hysteresis Erweitert den Regelzielwert in einen Bereich\r\n0: Regelzielwert wird versucht exakt zu erreichen\r\n>0: Regelzielwert +/- hysteresis wird versucht zu erreichen.
  '''
  def getHysteresis(self):
    return self.hysteresis






from pyhausbus.de.hausbus.homeassistant.proxy.counter.params.MMode import MMode
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 35
  FUNCTION_ID = 128

  def __init__(self,mode:MMode, debounceTime:int, reportTime:int, scaleFaktor:int):
    self.mode=mode
    self.debounceTime=debounceTime
    self.reportTime=reportTime
    self.scaleFaktor=scaleFaktor


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(MMode._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"Configuration(mode={self.mode}, debounceTime={self.debounceTime}, reportTime={self.reportTime}, scaleFaktor={self.scaleFaktor})"

  '''
  @param mode increment: 1 = Zaehler inkrementieren.
  '''
  def getMode(self) -> MMode:
    return self.mode

  '''
  @param debounceTime 0 - 255[ms].
  '''
  def getDebounceTime(self):
    return self.debounceTime

  '''
  @param reportTime Zeitintervall in Minuten nach dem der Zaehler den aktuellen Stand meldet.
  '''
  def getReportTime(self):
    return self.reportTime

  '''
  @param scaleFaktor Anzahl Impulse pro Einheit z.B. pro kWh.
  '''
  def getScaleFaktor(self):
    return self.scaleFaktor






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EErrorCode(Enum):
  NO_ERROR=0
  WRITE_FAILED=1
  READ_FAILED=2
  QUEUE_OVERRUN=3
  BUSY_BUS=4
  RESET_BUS=5
  BUFFER_OVERRUN=6
  NO_CONFIGURATION=7
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EErrorCode.__members__.values():
      if (act.value == checkValue):
        return act

    return EErrorCode.SER_UNKNOWN






from pyhausbus.WeekTime import WeekTime
import pyhausbus.HausBusUtils as HausBusUtils

class Weather:
  CLASS_ID = 2
  FUNCTION_ID = 128

  def __init__(self,humidity:int, pressure:int, temp:int, sunrise:WeekTime, sunset:WeekTime, text:str):
    self.humidity=humidity
    self.pressure=pressure
    self.temp=temp
    self.sunrise=sunrise
    self.sunset=sunset
    self.text=text


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Weather(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), WeekTime._fromBytes(dataIn, offset), WeekTime._fromBytes(dataIn, offset), HausBusUtils.bytesToString(dataIn, offset))

  def __str__(self):
    return f"Weather(humidity={self.humidity}, pressure={self.pressure}, temp={self.temp}, sunrise={self.sunrise}, sunset={self.sunset}, text={self.text})"

  '''
  @param humidity Luftfeuchtigkeit.
  '''
  def getHumidity(self):
    return self.humidity

  '''
  @param pressure Luftdruck.
  '''
  def getPressure(self):
    return self.pressure

  '''
  @param temp Temperatur.
  '''
  def getTemp(self):
    return self.temp

  '''
  @param sunrise Zeitpunkt vom Sonnenaufgang.
  '''
  def getSunrise(self):
    return self.sunrise

  '''
  @param sunset Zeitpunkt vom Sonnenuntergang.
  '''
  def getSunset(self):
    return self.sunset

  '''
  @param text Beschreibung.
  '''
  def getText(self):
    return self.text






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EState(Enum):
  OFF=0
  ON=1
  OFF_DELAY=2
  ON_DELAY=3
  TOGGLE=4
  DISABLED=5
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EState.__members__.values():
      if (act.value == checkValue):
        return act

    return EState.SER_UNKNOWN






from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.data.Status import Status
from pyhausbus.de.hausbus.homeassistant.proxy.powerMeter.params.ELastEvent import ELastEvent

class PowerMeter(ABusFeature):
  CLASS_ID:int = 41

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return PowerMeter(HausBusUtils.getObjectId(deviceId, 41, instanceId))

  """
  """
  def evLowPower(self):
    LOGGER.debug("evLowPower")
    hbCommand = HausBusCommand(self.objectId, 200, "evLowPower")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evMediumPower(self):
    LOGGER.debug("evMediumPower")
    hbCommand = HausBusCommand(self.objectId, 201, "evMediumPower")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evHighPower(self):
    LOGGER.debug("evHighPower")
    hbCommand = HausBusCommand(self.objectId, 202, "evHighPower")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param lowerThreshold untere Leistungsschwelle[kWh].
  @param lowerThresholdFraction Nachkommastellen der unteren Leistungsschwelle[00-99].
  @param upperThreshold obere Leistungsschwelle[kWh].
  @param upperThresholdFraction Nachkommastellen der oberen Leistungsschwelle[00-99].
  @param reportTimeBase Zeitbasis fuer die Einstellungen von minReportTime und maxReportTime.
  @param minReportTime Mindestzeit.
  @param maxReportTime Maximalzeit.
  @param hysteresis Hysterese [Wert * 0.
  @param calibration Dieser Wert wird verwendet um die vom Sensor gelieferten Messwerte zu justieren. [1/10 Prozent].
  @param deltaSensorID Die InstanceID des Sensors auf diesem Controller.
  """
  def setConfiguration(self, lowerThreshold:int, lowerThresholdFraction:int, upperThreshold:int, upperThresholdFraction:int, reportTimeBase:int, minReportTime:int, maxReportTime:int, hysteresis:int, calibration:int, deltaSensorID:int):
    LOGGER.debug("setConfiguration"+" lowerThreshold = "+str(lowerThreshold)+" lowerThresholdFraction = "+str(lowerThresholdFraction)+" upperThreshold = "+str(upperThreshold)+" upperThresholdFraction = "+str(upperThresholdFraction)+" reportTimeBase = "+str(reportTimeBase)+" minReportTime = "+str(minReportTime)+" maxReportTime = "+str(maxReportTime)+" hysteresis = "+str(hysteresis)+" calibration = "+str(calibration)+" deltaSensorID = "+str(deltaSensorID))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addSByte(lowerThreshold)
    hbCommand.addSByte(lowerThresholdFraction)
    hbCommand.addSByte(upperThreshold)
    hbCommand.addSByte(upperThresholdFraction)
    hbCommand.addByte(reportTimeBase)
    hbCommand.addByte(minReportTime)
    hbCommand.addByte(maxReportTime)
    hbCommand.addByte(hysteresis)
    hbCommand.addSByte(calibration)
    hbCommand.addByte(deltaSensorID)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getStatus(self):
    LOGGER.debug("getStatus")
    hbCommand = HausBusCommand(self.objectId, 2, "getStatus")
    ResultWorker()._setResultInfo(Status,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param lowerThreshold untere Leistungsschwelle[kWh].
  @param lowerThresholdFraction Nachkommastellen der unteren Leistungsschwelle[00-99].
  @param upperThreshold obere Leistungsschwelle[kWh].
  @param upperThresholdFraction Nachkommastellen der oberen Leistungsschwelle[00-99].
  @param reportTimeBase Zeitbasis fuer die Einstellungen von minReportTime und maxReportTime.
  @param minReportTime Mindestzeit.
  @param maxReportTime Maximalzeit.
  @param hysteresis Hysterese [Wert * 0.
  @param calibration Dieser Wert wird verwendet um die vom Sensor gelieferten Messwerte zu justieren. [1/10 Prozent].
  @param deltaSensorID Die InstanceID des Sensors auf diesem Controller.
  """
  def Configuration(self, lowerThreshold:int, lowerThresholdFraction:int, upperThreshold:int, upperThresholdFraction:int, reportTimeBase:int, minReportTime:int, maxReportTime:int, hysteresis:int, calibration:int, deltaSensorID:int):
    LOGGER.debug("Configuration"+" lowerThreshold = "+str(lowerThreshold)+" lowerThresholdFraction = "+str(lowerThresholdFraction)+" upperThreshold = "+str(upperThreshold)+" upperThresholdFraction = "+str(upperThresholdFraction)+" reportTimeBase = "+str(reportTimeBase)+" minReportTime = "+str(minReportTime)+" maxReportTime = "+str(maxReportTime)+" hysteresis = "+str(hysteresis)+" calibration = "+str(calibration)+" deltaSensorID = "+str(deltaSensorID))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addSByte(lowerThreshold)
    hbCommand.addSByte(lowerThresholdFraction)
    hbCommand.addSByte(upperThreshold)
    hbCommand.addSByte(upperThresholdFraction)
    hbCommand.addByte(reportTimeBase)
    hbCommand.addByte(minReportTime)
    hbCommand.addByte(maxReportTime)
    hbCommand.addByte(hysteresis)
    hbCommand.addSByte(calibration)
    hbCommand.addByte(deltaSensorID)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param power Aktueller Stromverbrauch.
  @param centiPower hundertstel Stromverbrauch.
  @param lastEvent .
  """
  def Status(self, power:int, centiPower:int, lastEvent:ELastEvent):
    LOGGER.debug("Status"+" power = "+str(power)+" centiPower = "+str(centiPower)+" lastEvent = "+str(lastEvent))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addSByte(power)
    hbCommand.addSByte(centiPower)
    hbCommand.addByte(lastEvent.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param power Stromverbrauch [kWh].
  @param centiPower hundertstel Stromverbrauch.
  @param lastEvent .
  """
  def evStatus(self, power:int, centiPower:int, lastEvent:ELastEvent):
    LOGGER.debug("evStatus"+" power = "+str(power)+" centiPower = "+str(centiPower)+" lastEvent = "+str(lastEvent))
    hbCommand = HausBusCommand(self.objectId, 203, "evStatus")
    hbCommand.addSByte(power)
    hbCommand.addSByte(centiPower)
    hbCommand.addByte(lastEvent.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import logging

from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EFirmwareId import (
    EFirmwareId,
)

LOGGER = logging.getLogger("pyhausbus")

device_type_map = {
    EFirmwareId.ESP32: {
        int("0x65", 16): "LAN-RS485 Brückenmodul",
        int("0x18", 16): "6-fach Taster",
        int("0x19", 16): "4-fach Taster",
        int("0x1A", 16): "2-fach Taster",
        int("0x1B", 16): "1-fach Taster",
        int("0x1C", 16): "6-fach Taster Gira",
        int("0x20", 16): "32-fach IO",
        int("0x0C", 16): "16-fach Relais",
        int("0x08", 16): "8-fach Relais",
        int("0x10", 16): "22-fach UP-IO",
        int("0x28", 16): "8-fach Dimmer",
        int("0x30", 16): "2-fach RGB Dimmer",
        int("0x00", 16): "4-fach 0-10V Dimmer",
        int("0x01", 16): "4-fach 1-10V Dimmer",
    },
    EFirmwareId.HBC: {
        int("0x18", 16): "6-fach Taster",
        int("0x19", 16): "4-fach Taster",
        int("0x1A", 16): "2-fach Taster",
        int("0x1B", 16): "1-fach Taster",
        int("0x1C", 16): "6-fach Taster Gira",
        int("0x20", 16): "32-fach IO",
        int("0x0C", 16): "16-fach Relais",
        int("0x08", 16): "8-fach Relais",
        int("0x10", 16): "24-fach UP-IO",
        int("0x28", 16): "8-fach Dimmer",
        int("0x29", 16): "8-fach Dimmer",
        int("0x30", 16): "2-fach RGB Dimmer",
        int("0x00", 16): "4-fach 0-10V Dimmer",
        int("0x01", 16): "4-fach 1-10V Dimmer",
    },
    EFirmwareId.SD485: {
        int("0x28", 16): "24-fach UP-IO",
        int("0x1E", 16): "6-fach Taster",
        int("0x2E", 16): "6-fach Taster",
        int("0x2F", 16): "6-fach Taster",
        int("0x2B", 16): "4-fach 0-10V Dimmer",
        int("0x2C", 16): "4-fach Taster",
        int("0x2D", 16): "4-fach 1-10V Dimmer",
        int("0x2A", 16): "2-fach Taster",
        int("0x29", 16): "1-fach Taster",
    },
    EFirmwareId.AR8: {
        int("0x28", 16): "LAN Brückenmodul",
        int("0x30", 16): "8-fach Relais",
    },
    EFirmwareId.SD6: {
        int("0x14", 16): "Multitaster",
        int("0x1E", 16): "Multitaster",
    },
}


def get_device_type(firmware_id: EFirmwareId, type_id: int) -> str:
    """Get device type from firmware ID and type ID."""
    firmware_model_id = device_type_map.get(firmware_id, {})
    if len(firmware_model_id) == 0:
        return "Controller"
    else:
        return firmware_model_id.get(type_id, "Controller")



from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.Status import Status
from pyhausbus.de.hausbus.homeassistant.proxy.schalter.params.EState import EState
from pyhausbus.de.hausbus.homeassistant.proxy.schalter.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.schalter.params.MOptions import MOptions
from pyhausbus.de.hausbus.homeassistant.proxy.schalter.data.Configuration import Configuration

class Schalter(ABusFeature):
  CLASS_ID:int = 19

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return Schalter(HausBusUtils.getObjectId(deviceId, 19, instanceId))

  """
  @param offTime Ausschaltdauer: \r\nWert * Zeitbasis [ms].
  @param onTime Einschaltdauer: \r\nWert * Zeitbasis [ms].
  @param quantity Anzahl der Zustandswechsel.
  """
  def toggle(self, offTime:int, onTime:int, quantity:int):
    LOGGER.debug("toggle"+" offTime = "+str(offTime)+" onTime = "+str(onTime)+" quantity = "+str(quantity))
    hbCommand = HausBusCommand(self.objectId, 4, "toggle")
    hbCommand.addByte(offTime)
    hbCommand.addByte(onTime)
    hbCommand.addByte(quantity)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param duration Einschaltdauer: \r\nWert * Zeitbasis [ms]\r\n0=nicht mehr ausschalten.
  @param onDelay Einschaltverzoegerung: Wert * Zeitbasis [ms]\r\n0=Keine.
  """
  def on(self, duration:int, onDelay:int):
    LOGGER.debug("on"+" duration = "+str(duration)+" onDelay = "+str(onDelay))
    hbCommand = HausBusCommand(self.objectId, 3, "on")
    hbCommand.addWord(duration)
    hbCommand.addWord(onDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param offDelay Ausschaltverzoegerung: Wert * Zeitbasis [ms]\r\n0=Keine.
  """
  def off(self, offDelay:int):
    LOGGER.debug("off"+" offDelay = "+str(offDelay))
    hbCommand = HausBusCommand(self.objectId, 2, "off")
    hbCommand.addWord(offDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param duration Dauer.
  """
  def evOn(self, duration:int):
    LOGGER.debug("evOn"+" duration = "+str(duration))
    hbCommand = HausBusCommand(self.objectId, 201, "evOn")
    hbCommand.addWord(duration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evOff(self):
    LOGGER.debug("evOff")
    hbCommand = HausBusCommand(self.objectId, 200, "evOff")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getStatus(self):
    LOGGER.debug("getStatus")
    hbCommand = HausBusCommand(self.objectId, 5, "getStatus")
    ResultWorker()._setResultInfo(Status,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state .
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos.
  """
  def Status(self, state:EState, duration:int):
    LOGGER.debug("Status"+" state = "+str(state)+" duration = "+str(duration))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addByte(state.value)
    hbCommand.addWord(duration)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evToggle(self):
    LOGGER.debug("evToggle")
    hbCommand = HausBusCommand(self.objectId, 202, "evToggle")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param maxOnTime Maximale Zeit.
  @param offDelayTime Verzoegerungszeit nach einem Off-Kommando.
  @param timeBase Zeitbasis [ms] fuer die Zeitabhaengigen Befehle.
  @param options Reservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  @param disableBitIndex Bit Index0-31 Systemvariable.
  """
  def Configuration(self, maxOnTime:int, offDelayTime:int, timeBase:int, options:MOptions, disableBitIndex:int):
    LOGGER.debug("Configuration"+" maxOnTime = "+str(maxOnTime)+" offDelayTime = "+str(offDelayTime)+" timeBase = "+str(timeBase)+" options = "+str(options)+" disableBitIndex = "+str(disableBitIndex))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(maxOnTime)
    hbCommand.addByte(offDelayTime)
    hbCommand.addWord(timeBase)
    hbCommand.addByte(options.getValue())
    hbCommand.addByte(disableBitIndex)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param maxOnTime Maximale Zeit.
  @param offDelayTime Verzoegerungszeit nach einem Off-Kommando.
  @param timeBase Zeitbasis [ms] fuer die Zeitabhaengigen Befehle.
  @param options Reservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  @param disableBitIndex Bit Index0-31 Systemvariable.
  """
  def setConfiguration(self, maxOnTime:int, offDelayTime:int, timeBase:int, options:MOptions, disableBitIndex:int):
    LOGGER.debug("setConfiguration"+" maxOnTime = "+str(maxOnTime)+" offDelayTime = "+str(offDelayTime)+" timeBase = "+str(timeBase)+" options = "+str(options)+" disableBitIndex = "+str(disableBitIndex))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(maxOnTime)
    hbCommand.addByte(offDelayTime)
    hbCommand.addWord(timeBase)
    hbCommand.addByte(options.getValue())
    hbCommand.addByte(disableBitIndex)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param cmdDelay Dauer Wert * Zeitbasis [ms].
  """
  def evCmdDelay(self, cmdDelay:int):
    LOGGER.debug("evCmdDelay"+" cmdDelay = "+str(cmdDelay))
    hbCommand = HausBusCommand(self.objectId, 203, "evCmdDelay")
    hbCommand.addWord(cmdDelay)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def evDisabled(self):
    LOGGER.debug("evDisabled")
    hbCommand = HausBusCommand(self.objectId, 204, "evDisabled")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.de.hausbus.homeassistant.proxy.ethernet.params.MOptions import MOptions
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 162
  FUNCTION_ID = 128

  def __init__(self,IP0:int, IP1:int, IP2:int, IP3:int, options:MOptions, Server_Port:int, Server_IP0:int, Server_IP1:int, Server_IP2:int, Server_IP3:int):
    self.IP0=IP0
    self.IP1=IP1
    self.IP2=IP2
    self.IP3=IP3
    self.options=options
    self.Server_Port=Server_Port
    self.Server_IP0=Server_IP0
    self.Server_IP1=Server_IP1
    self.Server_IP2=Server_IP2
    self.Server_IP3=Server_IP3


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), MOptions._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"Configuration(IP0={self.IP0}, IP1={self.IP1}, IP2={self.IP2}, IP3={self.IP3}, options={self.options}, Server_Port={self.Server_Port}, Server_IP0={self.Server_IP0}, Server_IP1={self.Server_IP1}, Server_IP2={self.Server_IP2}, Server_IP3={self.Server_IP3})"

  '''
  @param IP0 Eigene IP-Adresse im Format IP0.IP1.IP2.IP3.
  '''
  def getIP0(self):
    return self.IP0

  '''
  @param IP1 Eigene IP-Adresse im Format IP0.IP1.IP2.IP3.
  '''
  def getIP1(self):
    return self.IP1

  '''
  @param IP2 Eigene IP-Adresse im Format IP0.IP1.IP2.IP3.
  '''
  def getIP2(self):
    return self.IP2

  '''
  @param IP3 Eigene IP-Adresse im Format IP0.IP1.IP2.IP3.
  '''
  def getIP3(self):
    return self.IP3

  '''
  @param options .
  '''
  def getOptions(self) -> MOptions:
    return self.options

  '''
  @param Server_Port Zusaetzlicher Port fuer die Homeserverfunktionen z.B 15557 f?r Loxone oder 5855 f?r IOBroker.
  '''
  def getServer_Port(self):
    return self.Server_Port

  '''
  @param Server_IP0 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  '''
  def getServer_IP0(self):
    return self.Server_IP0

  '''
  @param Server_IP1 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  '''
  def getServer_IP1(self):
    return self.Server_IP1

  '''
  @param Server_IP2 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  '''
  def getServer_IP2(self):
    return self.Server_IP2

  '''
  @param Server_IP3 Server IP-Adresse im Format IP0.IP1.IP2.IP3 0.0.0.0 deaktiviert das Gateway 13 und 14.
  '''
  def getServer_IP3(self):
    return self.Server_IP3






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EReason(Enum):
  PowerOn=1
  External=2
  BrownOut=4
  Watchdog=8
  Debug=16
  Software=32
  Unknown=0
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EReason.__members__.values():
      if (act.value == checkValue):
        return act

    return EReason.SER_UNKNOWN



